package cn.iocoder.yudao.module.reward.controller.admin.redeem;

import cn.iocoder.yudao.framework.common.pojo.CommonResult;
import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.module.reward.api.reward.RedeemRewardApi;
import cn.iocoder.yudao.module.reward.api.reward.dto.RedeemRewardReqVO;
import cn.iocoder.yudao.module.reward.api.reward.dto.RedeemRewardRespDTO;
import cn.iocoder.yudao.module.reward.service.redeem.RewardRedeemService;
import cn.iocoder.yudao.module.reward.vo.reward.*;
import cn.iocoder.yudao.module.reward.vo.reward.RewardRedeemPageReqVO;
import cn.iocoder.yudao.module.reward.vo.reward.RewardRedeemPageRespVO;
import io.swagger.annotations.Api;
import io.swagger.annotations.ApiOperation;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.validation.annotation.Validated;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import javax.annotation.Resource;
import javax.validation.Valid;
import java.util.List;

import static cn.iocoder.yudao.framework.common.pojo.CommonResult.success;

@Validated
@RestController
@Api(tags = "管理后台 - 兑换记录管理")
@RequestMapping("/reward/redeem")
public class RedeemRewardController {

    @Resource
    private RedeemRewardApi redeemRewardApi;

    @Resource
    private RewardRedeemService rewardRedeemService;

    @PostMapping("record/page")
    @ApiOperation("列表")
    public CommonResult<PageResult<RewardRedeemPageRespVO>> pageInfo(@Valid @RequestBody RewardRedeemPageReqVO reqVO) {
        return success(rewardRedeemService.pageInfo(reqVO));
    }

    @PostMapping("record/detail")
    @ApiOperation("详情")
    public CommonResult<RewardRedeemPageRespVO> detail(@Valid @RequestBody RewardRedeemBaseVO request) {
        return success(rewardRedeemService.detail(request.getId()));
    }

    @PostMapping("record/update")
    @ApiOperation("更新")
    public CommonResult<Boolean> update(@Valid @RequestBody RewardRedeemUpdateReqVO request) {
        return success(rewardRedeemService.updateRecord(request));
    }

    @PostMapping("record/verify")
    @ApiOperation("核销")
    public CommonResult<Boolean> verify(@Valid @RequestBody RewardRedeemVerifyReqVO request) {
        return success(rewardRedeemService.verify(request));
    }

    @PostMapping("record/batch/verify")
    @ApiOperation("批量核销")
    public CommonResult<Boolean> batchVerify(@Valid @RequestBody RewardRedeemBatchVerifyReqVO request) {
        return success(rewardRedeemService.batchVerify(request));
    }

    @PostMapping("record/verify-back")
    @ApiOperation("核销回退")
    public CommonResult<Boolean> verifyBack(@Valid @RequestBody List<Long> redeemIds) {
        return success(rewardRedeemService.verifyBack(redeemIds));
    }

    @PostMapping("record/export")
    @ApiOperation("导出")
    public CommonResult<Boolean> export(@Valid @RequestBody RewardRedeemPageReqVO reqVO) {
        return success(rewardRedeemService.export(reqVO));
    }


    @PostMapping("/single")
    @ApiOperation("兑换礼品")
    @PreAuthorize("@ss.hasPermission('ecw:giftManagement:exchange')")
    public CommonResult<RedeemRewardRespDTO> redeemReward(@Valid @RequestBody RedeemRewardReqVO redeemRewardReqVO) {
        return success(redeemRewardApi.redeemReward(redeemRewardReqVO));
    }

    @PostMapping("/batch")
    @ApiOperation("批量兑换礼品")
    @PreAuthorize("@ss.hasPermission('ecw:giftManagement:exchange')")
    public CommonResult<List<RedeemRewardRespDTO>> redeemRewards(@Valid @RequestBody List<RedeemRewardReqVO> redeemRewardReqVOList) {
        return success(redeemRewardApi.redeemRewards(redeemRewardReqVOList));
    }
}
