package cn.iocoder.yudao.module.reward.service.redeem;

import cn.iocoder.yudao.framework.common.pojo.PageResult;
import cn.iocoder.yudao.framework.i18n.core.I18nMessage;
import cn.iocoder.yudao.framework.mybatis.core.service.AbstractService;
import cn.iocoder.yudao.framework.security.core.util.SecurityFrameworkUtils;
import cn.iocoder.yudao.module.reward.dal.dataobject.redeem.RewardRedeemDO;
import cn.iocoder.yudao.module.reward.dal.mysql.redeem.RewardRedeemMapper;
import cn.iocoder.yudao.module.reward.dto.RewardRedeemVerifyDTO;
import cn.iocoder.yudao.module.reward.enums.ErrorCodeConstants;
import cn.iocoder.yudao.module.reward.enums.RewardPickMethedEnum;
import cn.iocoder.yudao.module.reward.enums.RewardRedeemStatusEnum;
import cn.iocoder.yudao.module.reward.service.reward.RewardService;
import cn.iocoder.yudao.module.reward.vo.reward.*;
import cn.iocoder.yudao.module.system.api.file.FileMakeApi;
import cn.iocoder.yudao.module.system.api.file.dto.FileMakeReqDTO;
import com.alibaba.fastjson.JSONObject;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.toolkit.CollectionUtils;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import io.micrometer.core.instrument.util.StringUtils;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.Map;
import java.util.function.Function;
import java.util.stream.Collectors;

import static cn.iocoder.yudao.framework.common.exception.util.ServiceExceptionUtil.exception;
import static cn.iocoder.yudao.module.system.enums.download.DownloadTypeEnum.REWARD_REDEEM_RECORD;

/**
 * 礼品兑换 Service
 *
 * @author 系统管理员
 */
@Service
@Slf4j
public class RewardRedeemServiceImpl extends AbstractService<RewardRedeemMapper, RewardRedeemDO> implements RewardRedeemService {
    @Resource
    private RewardRedeemMapper rewardRedeemMapper;
    @Resource
    private RewardService rewardService;
    @Resource
    private FileMakeApi fileMakeApi;

    @Override
    public PageResult<RewardRedeemPageRespVO> pageInfo(RewardRedeemPageReqVO reqVO) {
        int start = (reqVO.getPageNo() - 1) * reqVO.getPageSize();
        int size = reqVO.getPageSize();
        List<RewardRedeemPageRespVO> result = rewardRedeemMapper.pageInfo(start, size, reqVO);
        int total = rewardRedeemMapper.pageCount(reqVO);
        return new PageResult<>(result, total, reqVO.getPageSize(), reqVO.getPageNo(), (total + reqVO.getPageSize() - 1) / reqVO.getPageSize());
    }

    @Override
    public RewardRedeemPageRespVO detail(Long id) {
        if (id == null) {
            return null;
        }
        return rewardRedeemMapper.detail(id);
    }

    @Override
    public Boolean updateRecord(RewardRedeemUpdateReqVO request) {
        updateValidateParam(request);
        LambdaUpdateWrapper<RewardRedeemDO> wrapper = Wrappers.lambdaUpdate();
        wrapper.eq(RewardRedeemDO::getId, request.getId());
        wrapper.set(RewardRedeemDO::getRedeemType, request.getRedeemType());
        wrapper.set(RewardRedeemDO::getExpenses, request.getExpenses());
        wrapper.set(RewardRedeemDO::getCurrency, request.getCurrency());
        wrapper.set(RewardRedeemDO::getRecipientName, request.getRecipientName());
        wrapper.set(RewardRedeemDO::getRecipientAddress, request.getRecipientAddress());
        wrapper.set(RewardRedeemDO::getRecipientPhoneNum, request.getRecipientPhoneNum());
        wrapper.set(RewardRedeemDO::getRedeemer, request.getRedeemer());
        wrapper.set(RewardRedeemDO::getRedemptionTime, request.getRedemptionTime());
        wrapper.set(RewardRedeemDO::getCourierCompany, request.getCourierCompany());
        wrapper.set(RewardRedeemDO::getExpressNo, request.getExpressNo());
        wrapper.set(RewardRedeemDO::getExpressDate, request.getExpressDate());
        wrapper.set(RewardRedeemDO::getExpressSender, request.getExpressSender());
        wrapper.set(RewardRedeemDO::getAnnex, request.getAnnex());
        wrapper.set(RewardRedeemDO::getRemark, request.getRemark());
        wrapper.set(RewardRedeemDO::getUpdater, SecurityFrameworkUtils.getLoginUserId());
        wrapper.set(RewardRedeemDO::getUpdateTime, new Date());
        rewardRedeemMapper.update(null, wrapper);
        return true;
    }

    @Override
    public Boolean verify(RewardRedeemVerifyReqVO request) {
        if (!checkVerifyParam(request)){
            throw exception(ErrorCodeConstants.REWARD_REDEEM_VERIFY_NO_PARAM);
        }
        LambdaUpdateWrapper<RewardRedeemDO> wrapper = Wrappers.lambdaUpdate();
        wrapper.eq(RewardRedeemDO::getId, request.getId());
        wrapper.set(RewardRedeemDO::getStatus, RewardRedeemStatusEnum.REDEEMED.getValue());
        wrapper.set(RewardRedeemDO::getRecipientName, request.getRecipientName());
        wrapper.set(RewardRedeemDO::getRecipientPhoneNum, request.getRecipientPhoneNum());
        wrapper.set(RewardRedeemDO::getRecipientAddress, request.getRecipientAddress());
        wrapper.set(RewardRedeemDO::getCourierCompany, request.getCourierCompany());
        wrapper.set(RewardRedeemDO::getExpressNo, request.getExpressNo());
        wrapper.set(RewardRedeemDO::getExpressDate, request.getExpressDate());
        wrapper.set(RewardRedeemDO::getExpressSender, request.getExpressSender());
        wrapper.set(RewardRedeemDO::getVerifyUser, request.getVerifyUser());
        wrapper.set(RewardRedeemDO::getVerifyTime, request.getVerifyTime());
        wrapper.set(RewardRedeemDO::getUpdater, SecurityFrameworkUtils.getLoginUserId());
        wrapper.set(RewardRedeemDO::getUpdateTime, new Date());
        rewardRedeemMapper.update(null, wrapper);
        return true;
    }

    @Override
    public Boolean batchVerify(RewardRedeemBatchVerifyReqVO request) {
        if (CollectionUtils.isEmpty(request.getIds())) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_NOT_EXIST);
        }
        List<Long> errorIds = checkBatchVerifyParam(request);
        if (CollectionUtils.isNotEmpty(errorIds)) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_BATCH_VERIFY_ERROR, errorIds);
        }
        LambdaUpdateWrapper<RewardRedeemDO> updateWrapper = Wrappers.lambdaUpdate();
        updateWrapper.in(RewardRedeemDO::getId, request.getIds());
        updateWrapper.set(RewardRedeemDO::getStatus, RewardRedeemStatusEnum.REDEEMED.getValue());
        updateWrapper.set(RewardRedeemDO::getVerifyUser, request.getVerifyUser());
        updateWrapper.set(RewardRedeemDO::getVerifyTime, request.getVerifyTime());
        updateWrapper.set(RewardRedeemDO::getUpdater, SecurityFrameworkUtils.getLoginUserId());
        updateWrapper.set(RewardRedeemDO::getUpdateTime, new Date());
        this.update(null, updateWrapper);
        return true;
    }

    @Override
    public Boolean verifyBack(List<Long> redeemIds) {
        if (CollectionUtils.isEmpty(redeemIds)) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_NOT_EXIST);
        }
        LambdaUpdateWrapper<RewardRedeemDO> wrapper = Wrappers.lambdaUpdate();
        wrapper.in(RewardRedeemDO::getId, redeemIds);
        List<RewardRedeemDO> list = this.list(wrapper);
        if (CollectionUtils.isEmpty(list)) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_NOT_EXIST);
        }
        List<RewardRedeemDO> statusErrorList = list.stream().filter(item -> item.getStatus() != RewardRedeemStatusEnum.REDEEMED.getValue())
                .collect(Collectors.toList());
        if (CollectionUtils.isNotEmpty(statusErrorList)) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_VERIFY_BACK_STATUS);
        }
        LambdaUpdateWrapper<RewardRedeemDO> updateWrapper = Wrappers.lambdaUpdate();
        updateWrapper.in(RewardRedeemDO::getId, redeemIds);
        updateWrapper.set(RewardRedeemDO::getStatus, RewardRedeemStatusEnum.REDEEMING.getValue());
        updateWrapper.set(RewardRedeemDO::getVerifyUser, null);
        updateWrapper.set(RewardRedeemDO::getVerifyTime, null);
        updateWrapper.set(RewardRedeemDO::getUpdater, SecurityFrameworkUtils.getLoginUserId());
        updateWrapper.set(RewardRedeemDO::getUpdateTime, new Date());
        this.update(null, updateWrapper);
        return true;
    }

    @Override
    public Boolean export(RewardRedeemPageReqVO reqVO) {
        FileMakeReqDTO fileMakeReqDTO = new FileMakeReqDTO();
        fileMakeReqDTO.setType(REWARD_REDEEM_RECORD.getType());
        fileMakeReqDTO.setName("礼品兑换记录导出excel");
        fileMakeReqDTO.setFileSuffix("xlsx");
        fileMakeReqDTO.setTemporaryFile(true);
        fileMakeReqDTO.setUserType(2);
        fileMakeReqDTO.setLang(I18nMessage.getLang());
        fileMakeReqDTO.setRequestParams(JSONObject.toJSONString(reqVO));
        fileMakeApi.sendFileMake(fileMakeReqDTO);
        return true;
    }

    @Override
    public List<RewardRedeemPageRespVO> exportList(RewardRedeemPageReqVO request) {
        return rewardRedeemMapper.exportList(request);
    }
    @Override
    public Integer exportCount(RewardRedeemPageReqVO request) {
        return rewardRedeemMapper.pageCount(request);
    }

    private List<Long> checkBatchVerifyParam(RewardRedeemBatchVerifyReqVO request) {
        LambdaUpdateWrapper<RewardRedeemDO> wrapper = Wrappers.lambdaUpdate();
        wrapper.in(RewardRedeemDO::getId, request.getIds());
        Map<Long, RewardRedeemDO> idRewardRedeemMap = this.list(wrapper).stream()
                .collect(Collectors.toMap(RewardRedeemDO::getId, Function.identity(), (c1, c2) -> c1));
        List<Long> errorIds = idRewardRedeemMap.keySet().stream().filter(id -> {
                    RewardRedeemDO rewardRedeemDO = idRewardRedeemMap.get(id);
                    if (rewardRedeemDO == null) {
                        return true;
                    }
                    return !canVerify(RewardRedeemVerifyDTO.builder()
                            .id(rewardRedeemDO.getId())
                            .status(rewardRedeemDO.getStatus())
                            .redeemType(rewardRedeemDO.getRedeemType())
                            .courierCompany(rewardRedeemDO.getCourierCompany())
                            .expressDate(rewardRedeemDO.getExpressDate())
                            .recipientAddress(rewardRedeemDO.getRecipientAddress())
                            .expressNo(rewardRedeemDO.getExpressNo())
                            .expressSender(rewardRedeemDO.getExpressSender())
                            .build());
                }).collect(Collectors.toList());
        return errorIds;
    }

    private boolean checkVerifyParam(RewardRedeemVerifyReqVO request) {
        RewardRedeemDO rewardRedeemDO = getById(request.getId());
        if (rewardRedeemDO == null) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_NOT_EXIST);
        }
        return canVerify(RewardRedeemVerifyDTO.builder()
                .id(rewardRedeemDO.getId())
                .status(rewardRedeemDO.getStatus())
                .redeemType(rewardRedeemDO.getRedeemType())
                .courierCompany(request.getCourierCompany())
                .expressDate(request.getExpressDate())
                .recipientAddress(request.getRecipientAddress())
                .expressNo(request.getExpressNo())
                .expressSender(request.getExpressSender())
                .build());
    }

    private boolean canVerify(RewardRedeemVerifyDTO rewardRedeemVerifyDTO) {
        if (rewardRedeemVerifyDTO.getStatus() != RewardRedeemStatusEnum.REDEEMING.getValue()) {
            return false;
        }
        if (rewardRedeemVerifyDTO.getRedeemType() != RewardPickMethedEnum.SELF_PICKUP.getValue()) {
            //除上门领取状态外，其他方式需要验证 courierCompany, expressNo, expressDate, expressSender, recipientAddress 不能为空
            if (rewardRedeemVerifyDTO.getCourierCompany() == null) {
                return false;
            }
            if (StringUtils.isBlank(rewardRedeemVerifyDTO.getExpressNo())) {
                return false;
            }
            if (rewardRedeemVerifyDTO.getExpressDate() == null) {
                return false;
            }
            if (StringUtils.isBlank(rewardRedeemVerifyDTO.getExpressSender())) {
                return false;
            }
            if (StringUtils.isBlank(rewardRedeemVerifyDTO.getRecipientAddress())) {
                return false;
            }
        }
        return true;
    }

    private void updateValidateParam(RewardRedeemUpdateReqVO request) {
        RewardRedeemDO rewardRedeemDO = getById(request.getId());
        if (rewardRedeemDO == null) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_NOT_EXIST);
        }
        if (rewardRedeemDO.getStatus() != RewardRedeemStatusEnum.REDEEMING.getValue()) {
            throw exception(ErrorCodeConstants.REWARD_REDEEM_STATUS_ERROR);
        }
    }
}
